
var POOL_NAMES = 
	['i', 'ii', 'iii', 'iv', 'v', 'vi', 'vii', 'viii', 'ix', 'x', 'xi', 'xii' ];

BC_1_8 =   ['AD001', 'AD002', 'AD003', 'AD004', 'AD005', 'AD006', 'AD007', 'AD008'];
BC_9_16 =  ['AD009', 'AD010', 'AD011', 'AD012', 'AD013', 'AD014', 'AD015', 'AD016'];
BC_18_27 = ['AD018', 'AD019', 'AD020', 'AD021', 'AD022', 'AD023', 'AD025', 'AD027'];

// Index Sets = IS
var IS_G1 = {color: 'bg-green',  barcodes: BC_1_8};
var IS_G2 = {color: 'bg-green',  barcodes: BC_9_16};
var IS_G3 = {color: 'bg-green',  barcodes: BC_18_27};

var IS_B1 = {color: 'bg-blue',   barcodes: BC_1_8};
var IS_B2 = {color: 'bg-blue',   barcodes: BC_9_16};
var IS_B3 = {color: 'bg-blue',   barcodes: BC_18_27};

var IS_Y1 = {color: 'bg-yellow', barcodes: BC_1_8};
var IS_Y2 = {color: 'bg-yellow', barcodes: BC_9_16};
var IS_Y3 = {color: 'bg-yellow', barcodes: BC_18_27};


var POOL_SCHEMA =
[
	{
		id: '4by24',
		name: '4 pools × 24 samples',
 		numPools: 4,
 		numWellsPerPool: 24,
 		columnClasses: ['pool-left', 'pool-middle', 'pool-right'],
		barcodeVariants:
			[
			 	{name: 'FULL', indexSets: [IS_G1, IS_B2, IS_Y3, IS_G1, IS_B2, IS_Y3, IS_G1, IS_B2, IS_Y3, IS_G1, IS_B2, IS_Y3]}
			],

		getPoolNumForColumn: function(colNum)
		{
			var poolNum = Math.floor(colNum / 3);
			return poolNum;
		},

 		getClassNameForWell: function(well)
		{
			var c = well.column;
			var cls = 'primary ';
			cls += this.columnClasses[c % 3];
			return cls;
		},
		
		getWellsInPool: function(plate, poolNum)
		{
			var result = [];
			var index = poolNum * this.numWellsPerPool;
			while (result.length < this.numWellsPerPool && index < plate.wells.length)
			{
				result[result.length] = plate.wells[index];
				index++;
			}
			return result;
		}
	
	},
	{
		id: '4by16',
		name: '4 pools × 16 samples',
 		numPools: 4,
 		numWellsPerPool: 16,
 		columnClasses: ['pool-left', 'pool-right'],
		barcodeVariants:
			[
			 	{name: 'A-B-C-A', indexSets: [null, null, IS_G1, IS_G2, IS_B3, IS_B1, IS_Y2, IS_Y3, IS_G1, IS_G2, null, null]},
			 	{name: 'B-C-A-B', indexSets: [null, null, IS_B3, IS_B1, IS_Y2, IS_Y3, IS_G1, IS_G2, IS_B3, IS_B1, null, null]},
			 	{name: 'C-A-B-C', indexSets: [null, null, IS_Y2, IS_Y3, IS_G1, IS_G2, IS_B3, IS_B1, IS_Y2, IS_Y3, null, null]}
			],

		getPoolNumForColumn: function(colNum)
		{
			var poolNum = -1;
			if (colNum >= 2 && colNum <= 9)
			{
				poolNum = Math.floor((colNum - 2) / 2);
			}
			return poolNum;
		},
 		
 		getClassNameForWell: function(well)
		{
			var c = well.column;
			var cls = (c >= 2 && c <= 9) ? 'primary ' : 'secondary ';
			cls += this.columnClasses[c % 2];
			return cls;
		},
		
		getWellsInPool: function(plate, poolNum)
 		{
 			var result = [];
 			var index = poolNum*this.numWellsPerPool + 2 * plate.rows;
			while (result.length < this.numWellsPerPool && index < plate.wells.length)
			{
				result[result.length] = plate.wells[index];
				index++;
			}
 			return result;
 		}
 	},
	{
		id: '5by16',
		name: '5 pools × 16 samples',
		numPools: 5,
		numWellsPerPool: 16,
		columnClasses: ['pool-right', 'pool-left'],
		barcodeVariants:
			[
			 	{name: 'A-B-C-A-B', indexSets: [null, IS_G1, IS_G2, IS_B3, IS_B1, IS_Y2, IS_Y3, IS_G1, IS_G2, IS_B3, IS_B1, null]},
			 	{name: 'B-C-A-B-C', indexSets: [null, IS_B3, IS_B1, IS_Y2, IS_Y3, IS_G1, IS_G2, IS_B3, IS_B1, IS_Y2, IS_Y3, null]},
			 	{name: 'C-A-B-C-A', indexSets: [null, IS_Y2, IS_Y3, IS_G1, IS_G2, IS_B3, IS_B1, IS_Y2, IS_Y3, IS_G1, IS_G2, null]}
			],
	
		getPoolNumForColumn: function(colNum)
		{
			var poolNum = -1;
			if (colNum >= 1 && colNum <= 10)
			{
				poolNum = Math.floor((colNum-1) / 2);
			}
			return poolNum;
		},
 		
		getClassNameForWell: function(well)
		{
			var c = well.column;
			var cls = (c >= 1 && c <= 10) ? 'primary ' : 'secondary ';
			cls += this.columnClasses[c % 2];
			return cls;
		},
		
		getWellsInPool: function(plate, poolNum)
 		{
 			var result = [];
 			var index = poolNum*this.numWellsPerPool+plate.rows;
			while (result.length < this.numWellsPerPool && index < plate.wells.length)
			{
				result[result.length] = plate.wells[index];
				index++;
			}
 			return result;
 		}
	}
];

/**
	Methods for handling pooling layout schemas.
*/
var PoolSchema = function()
{
	var ps = {};
	
	ps.initList = function(list, selectedId)
	{
		if (!selectedId) selectedId = POOL_SCHEMA[0].id;
		var selectedIndex = 0;
		for (var i = 0; i < POOL_SCHEMA.length; i++)
		{
			var schema = POOL_SCHEMA[i];
			var option = new Option(schema.name, schema.id);
			list[list.length] = option;
			if (selectedId == schema.id) selectedIndex = i;
		}
		list.selectedIndex = selectedIndex;
		return POOL_SCHEMA[selectedIndex];
	}
	
	ps.initVariantList = function(list, schema, selectedId)
	{
		if (!selectedId) selectedId = schema.barcodeVariants[0].name;
		var selectedIndex = 0;
		list.length = 0;
		for (var i = 0; i < schema.barcodeVariants.length; i++)
		{
			var variant = schema.barcodeVariants[i];
			var option = new Option(variant.name, variant.name);
			list[list.length] = option;
			if (selectedId == variant.name) selectedIndex = i;
		}
		list.selectedIndex = selectedIndex;
		return schema.barcodeVariants[selectedIndex];
	}
	
	ps.getById = function(schemaId)
	{
		for (var i = 0; i < POOL_SCHEMA.length; i++)
		{
			if (POOL_SCHEMA[i].id == schemaId)
			{
				return POOL_SCHEMA[i];
			}
		}
		return null;
	}
	
	ps.getBarcodeVariantByName = function(schema, barcodeVariant)
	{
		for (var i = 0; i < schema.barcodeVariants.length; i++)
		{
			if (barcodeVariant == schema.barcodeVariants[i].name) 
			{
				return schema.barcodeVariants[i];
			}
		}
		return schema.barcodeVariants[0];
	}
	
	ps.buildPoolTableRow = function(schema, columns, interactive)
	{
		var html = '<th></th>';
		var colNum = 0;
		var poolClass = interactive ? 'pool link' : 'pool';
		var afterPool = false;
		while (colNum < columns)
		{
			var poolNum = schema ? schema.getPoolNumForColumn(colNum) : -1;
			var colspan = 1;
			if (poolNum == -1)
			{
				var className = afterPool ? 'nopool-after-pool' : 'nopool';
				html += '<th class="'+className+'">&nbsp;</th>';
				var afterPool = false;
			}
			else
			{
				while (colNum+colspan < columns)
				{
					if (schema.getPoolNumForColumn(colNum+colspan) != poolNum) break;
					colspan++;
				}
				html += '<th id="pool.' + poolNum + '" colspan="'+colspan+'"';
				html += ' class="'+poolClass+'"';
				html += ' data-pool-num="' + poolNum + '"';
				html += '>'+POOL_NAMES[poolNum]+'</th>';
				afterPool = true;
			}
			colNum += colspan;
		}
		
		var row = document.getElementById('pool-row');
		row.innerHTML = html;
	}
	
	return ps;
}();


